package com.example.backend.movies.api;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.example.backend.categories.service.CategorieService;
import com.example.backend.core.configurations.Constants;
import com.example.backend.movies.model.MovieEntity;
import com.example.backend.movies.service.MovieService;

import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;

import jakarta.validation.Valid;

import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.PostMapping;

@Controller
@RequestMapping(MovieController.URL)
public class MovieController {

    public static final String URL = Constants.ADMIN_PREFIX + "/movies";
    private static final String MOVIE_VIEW = "movies";
    private static final String MOVIE_EDIT_VIEW = "movie-edit";
    private static final String PAGE_ATTRIBUTE = "page";
    private static final String MOVIE_ATTRIBUTE = "movie";
    private static final String CATEGORIEID_ATTRIBUTE = "categorieId";

    private final MovieService movieService;
    private final CategorieService categorieService;
    private final ModelMapper modelMapper;

    public MovieController(MovieService movieService, CategorieService categorieService, ModelMapper modelMapper) {
        this.modelMapper = modelMapper;
        this.categorieService = categorieService;
        this.movieService = movieService;
    }

    private MovieDTO toDto(MovieEntity entity) {
        return modelMapper.map(entity, MovieDTO.class);
    }

    private MovieEntity toEntity(MovieDTO dto) {
        final MovieEntity entity = modelMapper.map(dto, MovieEntity.class);
        entity.setCategorie(categorieService.get(dto.getCategorieId()));
        return entity;
    }

    @GetMapping()
    public String getAll(@RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        model.addAttribute("movies",
                movieService.getAll(0, page, Constants.DEFUALT_PAGE_SIZE)
                        .stream()
                        .map(this::toDto)
                        .toList());
        model.addAttribute("categories", categorieService.getAll());

        model.addAttribute(PAGE_ATTRIBUTE, page);
        model.addAttribute(CATEGORIEID_ATTRIBUTE, 0);
        return MOVIE_VIEW;
    }

    @GetMapping("/categorieId/{id}")
    public String getAll(@PathVariable(name = "id") Integer categorieId,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        model.addAttribute("movies",
                movieService.getAll(categorieId, page, Constants.DEFUALT_PAGE_SIZE)
                        .stream()
                        .map(this::toDto)
                        .toList());
        model.addAttribute("categories", categorieService.getAll());

        model.addAttribute(PAGE_ATTRIBUTE, page);
        model.addAttribute(CATEGORIEID_ATTRIBUTE, categorieId);
        return MOVIE_VIEW;
    }

    @GetMapping("/edit/")
    public String create(@RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page, Model model) {
        model.addAttribute("categories", categorieService.getAll());
        model.addAttribute(MOVIE_ATTRIBUTE, new MovieDTO());
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return MOVIE_EDIT_VIEW;
    }

    @PostMapping("/edit/")
    public String create(@RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @ModelAttribute(name = MOVIE_ATTRIBUTE) @Valid MovieDTO movieDTO,
            BindingResult bindingResult, Model model, RedirectAttributes redirectAttributes) {
        if (bindingResult.hasErrors()) {
            model.addAttribute(PAGE_ATTRIBUTE, page);
            return MOVIE_EDIT_VIEW;
        }
        model.addAttribute("categories", categorieService.getAll());
        model.addAttribute(CATEGORIEID_ATTRIBUTE, 0);
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        movieService.create(toEntity(movieDTO));
        return Constants.REDIRECT_VIEW + URL;
    }

    @GetMapping("/edit/{id}")
    public String update(@PathVariable(name = "id") Integer id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page, Model model) {
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        model.addAttribute("categories", categorieService.getAll());
        model.addAttribute(MOVIE_ATTRIBUTE, toDto(movieService.get(id)));
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return MOVIE_EDIT_VIEW;
    }

    @PostMapping("/edit/{id}")
    public String update(@PathVariable(name = "id") Integer id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @ModelAttribute(name = MOVIE_ATTRIBUTE) @Valid MovieDTO movieDTO,
            BindingResult bindingResult, Model model, RedirectAttributes redirectAttributes) {
        if (bindingResult.hasErrors()) {
            model.addAttribute(PAGE_ATTRIBUTE, page);
            return MOVIE_EDIT_VIEW;
        }
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        movieService.update(id, toEntity(movieDTO));
        return Constants.REDIRECT_VIEW + URL;
    }

    @PostMapping("/delete/{id}")
    public String delete(@PathVariable(name = "id") Integer id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            RedirectAttributes redirectAttributes) {
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        movieService.delete(id);
        return Constants.REDIRECT_VIEW + URL;
    }

    @GetMapping("/countView")
    public Integer countView(@RequestParam(name = "movieId", defaultValue = "0") Integer movieId) {
        return movieService.countView(movieId);
    }
}
