package com.example.backend.users.service;

import java.util.List;
import java.util.Optional;

import org.springframework.data.domain.PageRequest;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Sort;

import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.stream.StreamSupport;

import com.example.backend.core.configurations.Constants;
import com.example.backend.core.errors.NotFoundException;
import com.example.backend.core.security.UserPrincipal;
import com.example.backend.favorites.repository.FavoriteRepository;
import com.example.backend.users.model.UserEntity;
import com.example.backend.users.model.UserRole;
import com.example.backend.users.repository.UserRepository;
import com.example.backend.viewed.repository.ViewedRepository;

@Service
public class UserService implements UserDetailsService {

    private final UserRepository repository;
    private final PasswordEncoder passwordEncoder;
    private final FavoriteRepository favoriteRepository;
    private final ViewedRepository viewedRepository;

    public UserService(UserRepository repository, PasswordEncoder passwordEncoder,
            FavoriteRepository favoriteRepository,
            ViewedRepository viewedRepository) {
        this.repository = repository;
        this.passwordEncoder = passwordEncoder;
        this.favoriteRepository = favoriteRepository;
        this.viewedRepository = viewedRepository;
    }

    private void checkLogin(Integer id, String login) {
        final Optional<UserEntity> existsUser = repository.findByLoginIgnoreCase(login);
        if (existsUser.isPresent() && !existsUser.get().getId().equals(id)) {
            throw new IllegalArgumentException(
                    String.format("User with login %s is already exists", login));
        }
    }

    @Transactional(readOnly = true)
    public List<UserEntity> getAll() {
        return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
    }

    @Transactional(readOnly = true)
    public Page<UserEntity> getAll(int page, int size) {
        return repository.findAll(PageRequest.of(page, size, Sort.by("id")));
    }

    @Transactional(readOnly = true)
    public UserEntity get(Integer id) {
        return repository.findById(id).orElseThrow(() -> new NotFoundException(UserEntity.class, id));
    }

    @Transactional(readOnly = true)
    private UserEntity getByUsername(String username) {
        return repository.findByLoginIgnoreCase(username)
                .orElseThrow(() -> new IllegalArgumentException(
                        String.format("user with login %s is not found?", username)));
    }

    @Transactional
    public UserEntity create(UserEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }
        checkLogin(null, entity.getLogin());
        final String password = Optional.ofNullable(entity.getPassword()).orElse("");
        entity.setPassword(
                passwordEncoder.encode(StringUtils.hasText(password.strip()) ? password : Constants.DEFAULT_PASSWORD));
        entity.setRole(Optional.ofNullable(entity.getRole()).orElse(UserRole.USER));
        repository.save(entity);
        return repository.save(entity);
    }

    @Transactional
    public UserEntity update(Integer id, UserEntity entity) {
        final UserEntity existsentity = get(id);
        checkLogin(id, entity.getLogin());
        existsentity.setLogin(entity.getLogin());
        existsentity.setPassword(entity.getPassword());
        existsentity.setRole(entity.getRole());
        repository.save(existsentity);
        return existsentity;
    }

    @Transactional
    public UserEntity delete(Integer id) {
        final UserEntity existsentity = get(id);
        favoriteRepository.deleteByUserId(existsentity.getId());
        viewedRepository.deleteByUserId(existsentity.getId());

        repository.delete(existsentity);
        return existsentity;
    }

    @Transactional
    public Integer getCurrentUserId(String name) {
        final Optional<UserEntity> user = repository.findByLoginIgnoreCase(name);
        if (user.isEmpty()) {
            throw new IllegalArgumentException();
        }
        return user.get().getId();
    }

    @Override
    @Transactional(readOnly = true)
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        final UserEntity existUser = getByUsername(username);
        return new UserPrincipal(existUser);
    }

}
